'use strict';

exports.__esModule = true;

var _constants = require('./constants');

var _componentTracker = require('./componentTracker');

var componentTracker = _interopRequireWildcard(_componentTracker);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

/* eslint-env browser */
function loadState(rootState) {
  if (!rootState.children) return null;

  return Promise.all(rootState.children.map(function (state) {
    var component = componentTracker.get(state.id);

    if (!component) {
      console.warn('loadable-component client modules:', componentTracker.getAll());
      console.warn('loadable-component server modules:', window[_constants.LOADABLE_STATE]);
      throw new Error('loadable-components: module "' + state.id + '" is not found, client and server modules are not sync. You are probably not using the same resolver on server and client.');
    }

    var getLoadable = component[_constants.LOADABLE];

    if (typeof getLoadable !== 'function') {
      throw new Error('loadable-components: module "' + state.id + '" is not a laodable component, please verify your SSR setup');
    }

    return getLoadable().load().then(function () {
      return loadState(state);
    });
  }));
}

function loadComponents() {
  if (typeof window === 'undefined') {
    throw new Error('loadable-components: `loadComponents` must be called client-side: `window` is undefined');
  }

  var state = window[_constants.LOADABLE_STATE];
  if (!state) {
    throw new Error('loadable-components state not found. ' + 'You have a problem server-side. ' + 'Please verify that you have called `loadableState.getScriptTag()` server-side.');
  }

  return loadState(state);
}

exports.default = loadComponents;